@@ -13,6 +13,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
 #include <signal.h>
@@ -266,8 +267,21 @@ extern void pv_sig_init(pvstate_t);
  * Return true if SIGUSR2 has been received, and indicate the sender.
  */
 extern bool pv_sigusr2_received(pvstate_t, pid_t *);
+
+/*
+ * Return true if SIGUSR1 has been received, and indicate the sender.
+ */
+extern bool pv_sigusr1_received(pvstate_t, pid_t *);
 #endif
 
+/*
+ * Return a stream pointer for a control file associated with a particular
+ * process ID and signal.
+ */
+/*@null@ */
+/*@dependent@ */
+extern FILE *pv_open_controlfile(char *, size_t, pid_t, int, bool);
+
 /*
  * Enter the main transfer loop, transferring all input files to the output.
  */
@@ -278,6 +292,24 @@ extern int pv_main_loop(pvstate_t);
  */
 extern int pv_watchfd_loop(pvstate_t);
 
+/*
+ * Watch the progress of another pv process.
+ */
+extern int pv_query_loop(pvstate_t, pid_t);
+
+/*
+ * Set the options of another pv process.
+ */
+int pv_remote_set(pvstate_t, pid_t);
+
+/*
+ * Query another pv process for its elapsed transfer time, amount
+ * transferred, and total size, and update the local state with those
+ * values.  Optionally also return the total size separately.  Reports
+ * errors unless "silent" is true.
+ */
+int pv_remote_transferstate_fetch(pvstate_t state, pid_t query, /*@null@ */ off_t *sizeptr, bool silent);
+
 /*
  * Shut down signal handlers after running the main loop.
  */
