@@ -49,17 +49,53 @@
 #include <sys/ioctl.h>
 #endif
 
+/*
+ * The error prefix for messages (the program name); whether it has been
+ * set; and whether any output has been displayed yet, indicating whether
+ * any errors must be preceded by a newline.
+ */
+static char pv__error_prefix[64];	 /* flawfinder: ignore */
+static bool pv__error_prefix_set = false;
+static bool pv__output_produced = false;
+
+/*
+ * flawfinder rationale: zeroed before use, string copy is bounded to 1 less
+ * than size so it always has \0 termination.  Not used unless initialised,
+ * by checking pv__error_prefix_set.
+ */
+
+/*
+ * Set the error message prefix.
+ */
+void pv_set_error_prefix( /*@unique@ */ const char *prefix)
+{
+	if (NULL == prefix)
+		return;
+	memset(pv__error_prefix, 0, sizeof(pv__error_prefix));
+	strncpy(pv__error_prefix, prefix, sizeof(pv__error_prefix) - 1);	/* flawfinder: ignore */
+	pv__error_prefix_set = true;
+	/*
+	 * flawfinder rationale: strncpy's pointers are as valid as we can
+	 * make them since the first is a static buffer and the second is
+	 * caller-supplied.  The caller must \0-terminate the string but in
+	 * any case it is bounded to 1 less than the size of the
+	 * destination.  The destination is zeroed before use so the result
+	 * is guaranteed to be \0-terminated.
+	 */
+}
+
 /*
  * Output an error message.  If we've displayed anything to the terminal
  * already, then put a newline before our error so we don't write over what
  * we've written.
  */
-void pv_error(pvstate_t state, char *format, ...)
+void pv_error(char *format, ...)
 {
 	va_list ap;
-	if (state->display.display_visible)
+	if (pv__output_produced)
 		fprintf(stderr, "\n");
-	fprintf(stderr, "%s: ", state->status.program_name);
+	if (pv__error_prefix_set)
+		fprintf(stderr, "%s: ", pv__error_prefix);
 	va_start(ap, format);
 	(void) vfprintf(stderr, format, ap);	/* flawfinder: ignore */
 	va_end(ap);
@@ -140,9 +176,9 @@ void pv_write_retry(int fd, const char *buf, size_t count)
  * Write the given buffer to the terminal, like pv_write_retry(), unless
  * stderr is suspended.
  */
-void pv_tty_write(pvstate_t state, const char *buf, size_t count)
+void pv_tty_write(readonly_pvtransientflags_t flags, const char *buf, size_t count)
 {
-	while (0 == state->flag.suspend_stderr && count > 0) {
+	while (0 == flags->suspend_stderr && count > 0) {
 		ssize_t nwritten;
 
 		nwritten = write(STDERR_FILENO, buf, count);
@@ -553,15 +589,26 @@ char *pv_format_sequences(void)
 
 /*
  * Initialise the output format structure, based on the current options.
+ *
+ * May update status->checked_colour_support and
+ * status->terminal_supports_colour.
  */
-static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supplied, pvdisplay_t display)
+static void pv__format_init(pvprogramstatus_t status, readonly_pvcontrol_t control, readonly_pvtransferstate_t transfer,
+			    readonly_pvtransfercalc_t calc,
+			    /*@null@ */ const char *format_supplied, pvdisplay_t display)
 {
 	struct pvdisplay_component_s *format_component_array;
 	const char *display_format;
 	size_t strpos;
 	size_t segment;
 
-	if (NULL == state)
+	if (NULL == status)
+		return;
+	if (NULL == control)
+		return;
+	if (NULL == transfer)
+		return;
+	if (NULL == calc)
 		return;
 	if (NULL == display)
 		return;
@@ -578,7 +625,7 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 	display->showing_previous_line = false;
 	display->format_uses_colour = false;
 
-	display_format = NULL == format_supplied ? state->control.default_format : format_supplied;
+	display_format = NULL == format_supplied ? control->default_format : format_supplied;
 
 	if (NULL == display_format)
 		return;
@@ -779,9 +826,12 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 			memset(&formatter_info, 0, sizeof(formatter_info));
 			dummy_buffer[0] = '\0';
 
-			formatter_info.state = state;
 			formatter_info.display = display;
 			formatter_info.segment = &(display->format[segment]);
+			formatter_info.status = status;
+			formatter_info.control = control;
+			formatter_info.transfer = transfer;
+			formatter_info.calc = calc;
 			formatter_info.buffer = dummy_buffer;
 			formatter_info.buffer_size = 0;
 			formatter_info.offset = 0;
@@ -801,16 +851,16 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 		display->format_segment_count++;
 	}
 
-	if (display->format_uses_colour && !state->control.checked_colour_support) {
-		state->control.checked_colour_support = true;
+	if (display->format_uses_colour && !status->checked_colour_support) {
+		status->checked_colour_support = true;
 #ifdef ENABLE_NCURSES
 		/*
 		 * If we have terminal info support, check whether the
 		 * current terminal supports colour - or just assume it's
 		 * supported if we're forcing output.
 		 */
-		if (true == state->control.force) {
-			state->control.can_display_colour = true;
+		if (true == control->force) {
+			status->terminal_supports_colour = true;
 			debug("%s", "force mode - assuming terminal supports colour");
 		} else {
 			char *term_env = NULL;
@@ -820,22 +870,22 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 			 * flawfinder - here we pass responsibility to the
 			 * ncurses library to behave OK with $TERM.
 			 */
-			state->control.can_display_colour = false;
+			status->terminal_supports_colour = false;
 			if (NULL != term_env) {
 				int setup_err = 0;
 
 				if ((0 == setupterm(term_env, STDERR_FILENO, &setup_err))
 				    && (tigetnum("colors") > 1)
 				    ) {
-					state->control.can_display_colour = true;
+					status->terminal_supports_colour = true;
 					debug("%s: %s", term_env, "terminal supports colour");
 				} else {
-					state->control.can_display_colour = false;
+					status->terminal_supports_colour = false;
 					debug("%s: %s", term_env, "terminal does not support colour");
 				}
 			} else {
 				/* If TERM is unset, disable colour. */
-				state->control.can_display_colour = false;
+				status->terminal_supports_colour = false;
 				debug("%s", "no TERM variable - disabling colour support");
 			}
 		}
@@ -846,8 +896,8 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 		 * to determine whether colour is available, unless --force
 		 * was supplied, in which case colour support is assumed.
 		 */
-		if (true == state->control.force) {
-			state->control.can_display_colour = true;
+		if (true == control->force) {
+			status->terminal_supports_colour = true;
 			debug("%s", "force mode - assuming terminal supports colour");
 		} else {
 			FILE *command_fptr;
@@ -863,21 +913,21 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 			 */
 
 			if (NULL == command_fptr) {
-				state->control.can_display_colour = false;
+				status->terminal_supports_colour = false;
 				debug("%s (%s)", "popen failed - disabling colour support", strerror(errno));
 			} else {
 				int colour_count;
 				if (1 == fscanf(command_fptr, "%d", &colour_count)) {
 					if (colour_count > 1) {
-						state->control.can_display_colour = true;
+						status->terminal_supports_colour = true;
 						debug("%s (%d)", "terminal supports colour", colour_count);
 					} else {
-						state->control.can_display_colour = false;
+						status->terminal_supports_colour = false;
 						debug("%s (%d)",
 						      "fewer than 2 colours available - disabling colour support");
 					}
 				} else {
-					state->control.can_display_colour = false;
+					status->terminal_supports_colour = false;
 					debug("%s", "tput did not produce a number - disabling colour support");
 				}
 				/*@-unrecog@ *//* splint doesn't know pclose(). */
@@ -890,7 +940,7 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 		 * Without terminal info support, just assume colour is
 		 * available.
 		 */
-		state->control.can_display_colour = true;
+		status->terminal_supports_colour = true;
 		debug("%s", "terminal info support not compiled in - assuming colour support");
 #endif				/* (! ENABLE_NCURSES) && (! USE_POPEN_TPUTS) */
 #endif				/* ! ENABLE_NCURSES */
@@ -900,7 +950,7 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
 
 /*
  * Update display->display_buffer with status information formatted
- * according to the state held within the given structure.
+ * according to the state held within the given structures.
  *
  * If "reinitialise" is true, the format string is reparsed first.  This
  * should be true for the first call, and true whenever the format is
@@ -914,9 +964,14 @@ static void pv__format_init(pvstate_t state, /*@null@ */ const char *format_supp
  * When returning true, this function will have also set
  * display->display_string_len to the length of the string in
  * display->display_buffer, in bytes.
+ *
+ * Updates status->exit_status if buffer allocation fails.
+ *
+ * See pv__format_init for the adjustments that may be made to "status".
  */
-bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdisplay_t display, bool reinitialise,
-	       bool final)
+bool pv_format(pvprogramstatus_t status, readonly_pvcontrol_t control, readonly_pvtransferstate_t transfer,
+	       readonly_pvtransfercalc_t calc, /*@null@ */ const char *format_supplied, pvdisplay_t display,
+	       bool reinitialise, bool final)
 {
 	struct pvdisplay_component_s *format_component_array;
 	char display_segments[PV_SIZEOF_FORMAT_SEGMENTS_BUF];	/* flawfinder: ignore - always bounded */
@@ -930,17 +985,26 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 	memset(&formatter_info, 0, sizeof(formatter_info));
 	display_segments[0] = '\0';
 
-	/* Quick safety check - state and display must exist. */
-	if (NULL == state)
+	/* Quick safety check for null pointers. */
+	if (NULL == status)
+		return false;
+	if (NULL == control)
+		return false;
+	if (NULL == transfer)
+		return false;
+	if (NULL == calc)
 		return false;
 	if (NULL == display)
 		return false;
 
-	formatter_info.state = state;
 	formatter_info.display = display;
 	formatter_info.buffer = display_segments;
 	formatter_info.buffer_size = sizeof(display_segments);
 	formatter_info.offset = 0;
+	formatter_info.status = status;
+	formatter_info.control = control;
+	formatter_info.transfer = transfer;
+	formatter_info.calc = calc;
 
 	format_component_array = pv__format_components();
 
@@ -949,24 +1013,24 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 
 	/* Reinitialise if we were asked to. */
 	if (reinitialise)
-		pv__format_init(state, format_supplied, display);
+		pv__format_init(status, control, transfer, calc, format_supplied, display);
 
 	/* The format string is needed for the static segments. */
-	display_format = NULL == format_supplied ? state->control.default_format : format_supplied;
+	display_format = NULL == format_supplied ? control->default_format : format_supplied;
 	if (NULL == display_format)
 		return false;
 
 	/* Determine the type of thing being counted for transfer, rate, etc. */
 	display->count_type = PV_TRANSFERCOUNT_BYTES;
-	if (state->control.linemode)
+	if (control->linemode)
 		display->count_type = PV_TRANSFERCOUNT_LINES;
-	else if (state->control.decimal_units)
+	else if (control->decimal_units)
 		display->count_type = PV_TRANSFERCOUNT_DECBYTES;
 
 	/*
 	 * Reallocate the output buffer if the display width changes.
 	 */
-	if (display->display_buffer != NULL && display->display_buffer_size < (size_t) ((state->control.width * 4))) {
+	if (display->display_buffer != NULL && display->display_buffer_size < (size_t) ((control->width * 4))) {
 		free(display->display_buffer);
 		display->display_buffer = NULL;
 		display->display_buffer_size = 0;
@@ -979,15 +1043,15 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 		char *new_buffer;
 		size_t new_size;
 
-		new_size = (size_t) ((4 * state->control.width) + 80);
-		if (NULL != state->control.name)
-			new_size += strlen(state->control.name);	/* flawfinder: ignore */
+		new_size = (size_t) ((4 * control->width) + 80);
+		if (NULL != control->name)
+			new_size += strlen(control->name);	/* flawfinder: ignore */
 		/* flawfinder: name is always set by pv_strdup(), which bounds with a \0. */
 
 		new_buffer = malloc(new_size + 16);
 		if (NULL == new_buffer) {
-			pv_error(state, "%s: %s", _("buffer allocation failed"), strerror(errno));
-			state->status.exit_status |= PV_ERROREXIT_MEMORY;
+			pv_error("%s: %s", _("buffer allocation failed"), strerror(errno));
+			status->exit_status |= PV_ERROREXIT_MEMORY;
 			display->display_buffer = NULL;
 			return false;
 		}
@@ -1054,8 +1118,8 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 	 */
 
 	dynamic_segment_width = 0;
-	if (state->control.width > static_portion_width)
-		dynamic_segment_width = state->control.width - static_portion_width;
+	if (control->width > static_portion_width)
+		dynamic_segment_width = control->width - static_portion_width;
 
 	/*
 	 * Divide the total remaining screen space by the number of dynamic
@@ -1064,8 +1128,8 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 	if (dynamic_segment_count > 1)
 		dynamic_segment_width /= dynamic_segment_count;
 
-	debug("control.width=%d static_portion_width=%d dynamic_segment_width=%d dynamic_segment_count=%d",
-	      state->control.width, static_portion_width, dynamic_segment_width, dynamic_segment_count);
+	debug("control->width=%d static_portion_width=%d dynamic_segment_width=%d dynamic_segment_count=%d",
+	      control->width, static_portion_width, dynamic_segment_width, dynamic_segment_count);
 
 	for (segment_idx = 0; segment_idx < display->format_segment_count; segment_idx++) {
 		pvdisplay_segment_t segment;
@@ -1153,7 +1217,7 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 	 * spaces at the end, so that we don't leave dangling bits behind.
 	 */
 	if ((new_display_string_width < display->display_string_width)
-	    && (state->control.width >= display->prev_screen_width)) {
+	    && (control->width >= display->prev_screen_width)) {
 		char spaces[32];	 /* flawfinder: ignore - terminated, bounded */
 		int spaces_to_add;
 
@@ -1173,7 +1237,7 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
 
 	display->display_string_bytes = new_display_string_bytes;
 	display->display_string_width = new_display_string_width;
-	display->prev_screen_width = state->control.width;
+	display->prev_screen_width = control->width;
 
 	return true;
 }
@@ -1185,78 +1249,98 @@ bool pv_format(pvstate_t state, /*@null@ */ const char *format_supplied, pvdispl
  * If "final" is true, this is the final update, so the rate is given as an
  * an average over the whole transfer; otherwise the current rate is shown.
  */
-void pv_display(pvstate_t state, bool final)
+void pv_display(pvprogramstatus_t status, readonly_pvcontrol_t control, pvtransientflags_t flags,
+		readonly_pvtransferstate_t transfer, pvtransfercalc_t calc, pvcursorstate_t cursor, pvdisplay_t display,
+		/*@null@ */ pvdisplay_t extra_display, bool final)
 {
 	bool reinitialise = false;
 
-	if (NULL == state)
+	if (NULL == status)
+		return;
+	if (NULL == control)
+		return;
+	if (NULL == flags)
+		return;
+	if (NULL == transfer)
+		return;
+	if (NULL == calc)
+		return;
+	if (NULL == cursor)
+		return;
+	if (NULL == display)
 		return;
 
 	pv_sig_checkbg();
 
-	pv_calculate_transfer_rate(state, final);
+	pv_calculate_transfer_rate(calc, transfer, control, display, final);
 
 	/*
 	 * Enable colour on the main display, and disable it on the extra
 	 * display (process title, window title).
 	 */
-	state->display.colour_permitted = true;
-	state->extra_display.colour_permitted = false;
+	display->colour_permitted = true;
+	if (NULL != extra_display)
+		extra_display->colour_permitted = false;
 
 	/*
 	 * If the display options need reparsing, do so to generate new
 	 * formatting parameters.
 	 */
-	if (0 != state->flag.reparse_display) {
+	if (0 != flags->reparse_display) {
 		reinitialise = true;
-		state->flag.reparse_display = 0;
+		flags->reparse_display = 0;
 	}
 
-	if (!pv_format(state, state->control.format_string, &(state->display), reinitialise, final))
+	if (!pv_format(status, control, transfer, calc, control->format_string, display, reinitialise, final))
 		return;
 
-	if (0 != state->control.extra_displays) {
-		if (!pv_format(state, state->control.extra_format_string, &(state->extra_display), reinitialise, final))
+	if ((NULL != extra_display) && (0 != control->extra_displays)) {
+		if (!pv_format
+		    (status, control, transfer, calc, control->extra_format_string, extra_display, reinitialise, final))
 			return;
 	}
 
-	if (NULL == state->display.display_buffer)
+	if (NULL == display->display_buffer)
 		return;
 
-	if (state->control.numeric) {
-		pv_tty_write(state, state->display.display_buffer, state->display.display_string_bytes);
-		pv_tty_write(state, "\n", 1);
-	} else if (state->control.cursor) {
-		if (state->control.force || pv_in_foreground()) {
-			pv_crs_update(state, state->display.display_buffer);
-			state->display.display_visible = true;
+	if (control->numeric) {
+		pv_tty_write(flags, display->display_buffer, display->display_string_bytes);
+		pv_tty_write(flags, "\n", 1);
+	} else if (control->cursor) {
+		if (control->force || pv_in_foreground()) {
+			pv_crs_update(cursor, control, flags, display->display_buffer);
+			display->output_produced = true;
+			pv__output_produced = true;
 		}
 	} else {
-		if (state->control.force || pv_in_foreground()) {
-			pv_tty_write(state, state->display.display_buffer, state->display.display_string_bytes);
-			pv_tty_write(state, "\r", 1);
-			state->display.display_visible = true;
+		if (control->force || pv_in_foreground()) {
+			pv_tty_write(flags, display->display_buffer, display->display_string_bytes);
+			pv_tty_write(flags, "\r", 1);
+			display->output_produced = true;
+			pv__output_produced = true;
 		}
 	}
 
-	debug("%s: [%s]", "display", state->display.display_buffer);
+	debug("%s: [%s]", "display", display->display_buffer);
 
-	if ((0 != (PV_DISPLAY_WINDOWTITLE & state->control.extra_displays))
-	    && (state->control.force || pv_in_foreground())
-	    && (NULL != state->extra_display.display_buffer)
+	if ((0 != (PV_DISPLAY_WINDOWTITLE & control->extra_displays))
+	    && (control->force || pv_in_foreground())
+	    && (NULL != extra_display)
+	    && (NULL != extra_display->display_buffer)
 	    ) {
-		pv_tty_write(state, "\033]2;", 4);
-		pv_tty_write(state, state->extra_display.display_buffer, state->extra_display.display_string_bytes);
-		pv_tty_write(state, "\033\\", 2);
-		state->extra_display.display_visible = true;
-		debug("%s: [%s]", "windowtitle display", state->extra_display.display_buffer);
+		pv_tty_write(flags, "\033]2;", 4);
+		pv_tty_write(flags, extra_display->display_buffer, extra_display->display_string_bytes);
+		pv_tty_write(flags, "\033\\", 2);
+		extra_display->output_produced = true;
+		debug("%s: [%s]", "windowtitle display", extra_display->display_buffer);
 	}
 
-	if ((0 != (PV_DISPLAY_PROCESSTITLE & state->control.extra_displays))
-	    && (NULL != state->extra_display.display_buffer)
+	if ((0 != (PV_DISPLAY_PROCESSTITLE & control->extra_displays))
+	    && (NULL != extra_display)
+	    && (NULL != extra_display->display_buffer)
 	    ) {
-		setproctitle("%s", state->extra_display.display_buffer);
-		state->extra_display.display_visible = true;
-		debug("%s: [%s]", "processtitle display", state->extra_display.display_buffer);
+		setproctitle("%s -- %s", PACKAGE_NAME, extra_display->display_buffer);
+		extra_display->output_produced = true;
+		debug("%s: [%s]", "processtitle display", extra_display->display_buffer);
 	}
 }
