@@ -23,16 +23,15 @@ pvdisplay_bytecount_t pv_formatter_buffer_percent(pvformatter_args_t args)
 	if (0 == args->buffer_size)
 		return 0;
 
-	if (args->state->transfer.buffer_size > 0) {
+	if (args->transfer->buffer_size > 0) {
 		double pct_used = pv_percentage((off_t)
-						(args->state->transfer.read_position -
-						 args->state->transfer.write_position),
+						(args->transfer->read_position - args->transfer->write_position),
 						(off_t)
-						(args->state->transfer.buffer_size));
+						(args->transfer->buffer_size));
 		(void) pv_snprintf(content, sizeof(content), "{%3.0f%%}", pct_used);
 	}
 #ifdef HAVE_SPLICE
-	if (args->state->transfer.splice_used)
+	if (args->transfer->splice_used)
 		(void) pv_snprintf(content, sizeof(content), "{%s}", "----");
 #endif
 
