@@ -26,20 +26,17 @@ pvdisplay_bytecount_t pv_formatter_bytes(pvformatter_args_t args)
 	content[0] = '\0';
 
 	/*@-mustfreefresh@ */
-	if (args->state->control.numeric) {
+	if (args->control->numeric) {
 		/* Numeric mode - raw values only, no suffix. */
 		(void) pv_snprintf(content, sizeof(content),
-				   "%lld",
-				   (long long) ((args->state->control.bits ? 8 : 1) *
-						args->state->transfer.transferred));
-	} else if (args->state->control.bits && !args->state->control.linemode) {
+				   "%lld", (long long) ((args->control->bits ? 8 : 1) * args->transfer->transferred));
+	} else if (args->control->bits && !args->control->linemode) {
 		pv_describe_amount(content, sizeof(content), "%s",
-				   (long double) (args->state->transfer.transferred * 8), "", _("b"),
+				   (long double) (args->transfer->transferred * 8), "", _("b"),
 				   args->display->count_type);
 	} else {
 		pv_describe_amount(content, sizeof(content), "%s",
-				   (long double) (args->state->transfer.transferred), "", _("B"),
-				   args->display->count_type);
+				   (long double) (args->transfer->transferred), "", _("B"), args->display->count_type);
 	}
 	/*@+mustfreefresh@ *//* splint - false positive from gettext(). */
 
