@@ -35,8 +35,8 @@ pvdisplay_bytecount_t pv_formatter_name(pvformatter_args_t args)
 	(void) pv_snprintf(string_format, sizeof(string_format), "%%%d.500s:", field_width);
 
 	content[0] = '\0';
-	if (args->state->control.name) {
-		(void) pv_snprintf(content, sizeof(content), string_format, args->state->control.name);
+	if (NULL != args->control->name) {
+		(void) pv_snprintf(content, sizeof(content), string_format, args->control->name);
 	}
 
 	return pv_formatter_segmentcontent(content, args);
